@@ -196,7 +196,7 @@ static void manage_gensec_get_pw_request(enum stdio_helper_mode stdio_helper_mod
 {
 	DATA_BLOB in;
 	if (strlen(buf) < 2) {
-		DEBUG(1, ("query [%s] invalid", buf));
+		DEBUG(1, ("query [%s] invalid\n", buf));
 		printf("BH Query invalid\n");
 		return;
 	}
@@ -731,6 +731,10 @@ static NTSTATUS contact_winbind_change_pswd_auth_crap(const char *username,
     return nt_status;
 }
 
+/*
+ * This function does not create a full auth_session_info, just enough
+ * for the caller to get the "unix" username
+ */
 static NTSTATUS ntlm_auth_generate_session_info(struct auth4_context *auth_context,
 						TALLOC_CTX *mem_ctx,
 						void *server_returned_info,
@@ -759,7 +763,17 @@ static NTSTATUS ntlm_auth_generate_session_info(struct auth4_context *auth_conte
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	session_info->security_token = talloc_zero(session_info, struct security_token);
+	/*
+	 * This is not a full session_info - it is not created
+	 * correctly and misses any claims etc, because all we
+	 * actually use in the caller is the unix username.
+	 *
+	 * Therefore so no claims need to be added and
+	 * se_access_check() will never run.
+	 */
+	session_info->security_token
+		= security_token_initialise(talloc_tos(),
+					    CLAIMS_EVALUATION_INVALID_STATE);
 	if (session_info->security_token == NULL) {
 		TALLOC_FREE(session_info);
 		return NT_STATUS_NO_MEMORY;
@@ -1484,7 +1498,7 @@ static void manage_gensec_request(enum stdio_helper_mode stdio_helper_mode,
 	}
 
 	if (strlen(buf) < 2) {
-		DEBUG(1, ("query [%s] invalid", buf));
+		DEBUG(1, ("query [%s] invalid\n", buf));
 		printf("BH Query invalid\n");
 		talloc_free(mem_ctx);
 		return;
@@ -2701,7 +2715,7 @@ enum {
 			opt_challenge = strhex_to_data_blob(NULL, hex_challenge);
 			if (opt_challenge.length != 8) {
 				fprintf(stderr, "hex decode of %s failed! "
-					"(got %d bytes, expected 8)\n",
+					"(only got %d bytes)\n",
 					hex_challenge,
 					(int)opt_challenge.length);
 				exit(1);
@@ -2711,7 +2725,7 @@ enum {
 			opt_lm_response = strhex_to_data_blob(NULL, hex_lm_response);
 			if (opt_lm_response.length != 24) {
 				fprintf(stderr, "hex decode of %s failed! "
-					"(got %d bytes, expected 24)\n",
+					"(only got %d bytes)\n",
 					hex_lm_response,
 					(int)opt_lm_response.length);
 				exit(1);
@@ -2722,7 +2736,7 @@ enum {
 			opt_nt_response = strhex_to_data_blob(NULL, hex_nt_response);
 			if (opt_nt_response.length < 24) {
 				fprintf(stderr, "hex decode of %s failed! "
-					"(only got %d bytes, needed at least 24)\n",
+					"(only got %d bytes)\n",
 					hex_nt_response,
 					(int)opt_nt_response.length);
 				exit(1);
@@ -2835,7 +2849,7 @@ enum {
 	}
 
 	/* Exit code */
-
+	gfree_all();
 	poptFreeContext(pc);
 	TALLOC_FREE(frame);
 	return 0;
